/*
 * Copyright (C) 2004 Jimmy Do <crispyleaves@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "start-timer-dialog.h"
#include "profile-manager.h"
#include "add-profile-dialog.h"
#include "layout-utils.h"

static void
dialog_set_duration (GtkWidget *start_timer_dialog, gdouble duration)
{
	GtkWidget *spin_hours, *spin_minutes, *spin_seconds;
	guint num_hours, num_minutes, num_seconds;
	
	g_assert (start_timer_dialog);
	g_assert (duration >= 0);
	
	spin_hours = g_object_get_data (G_OBJECT (start_timer_dialog), "spin-hours");
	spin_minutes = g_object_get_data (G_OBJECT (start_timer_dialog), "spin-minutes");
	spin_seconds = g_object_get_data (G_OBJECT (start_timer_dialog), "spin-seconds");
	
	num_hours = (guint)(duration / 3600);
	num_minutes = (guint)((duration - (num_hours * 3600)) / 60);
	num_seconds = (guint)(duration - (num_hours * 3600) - (num_minutes * 60));

	g_assert (num_hours >= 0);
	g_assert (num_minutes >= 0);
	g_assert (num_seconds >= 0);
	g_assert (num_hours <= MAX_NUM_HOURS);
	g_assert (num_minutes <= 59);
	g_assert (num_seconds <= 59);

	gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_hours), num_hours);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_minutes), num_minutes);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_seconds), num_seconds);
}

static void
handle_ok_clicked (TimerApplet *timer_applet)
{
	gchar *preset_name;
	gdouble timer_duration;
	gboolean radio_instant_active;

	g_assert (timer_applet);
	g_assert (timer_applet->start_timer_dialog);
	
	gtk_widget_hide (timer_applet->start_timer_dialog);

	preset_name = NULL;
	
	g_object_get (G_OBJECT (timer_applet->start_timer_radio_instant), "active", &radio_instant_active, NULL);
	if (radio_instant_active) {
		GtkWidget *spin_hours, *spin_minutes, *spin_seconds;
		gint num_hours, num_minutes, num_seconds;
		GtkWidget *start_timer_dialog;

		start_timer_dialog = timer_applet->start_timer_dialog;
		
		spin_hours = g_object_get_data (G_OBJECT (start_timer_dialog), "spin-hours");
		spin_minutes = g_object_get_data (G_OBJECT (start_timer_dialog), "spin-minutes");
		spin_seconds = g_object_get_data (G_OBJECT (start_timer_dialog), "spin-seconds");
		
		num_hours = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin_hours));
		num_minutes = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin_minutes));
		num_seconds = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin_seconds));

		timer_duration = num_hours * 3600 + num_minutes * 60 + num_seconds;
	}
	else
	{
		GtkTreeRowReference *selected_row;
		
		selected_row = profile_manager_get_selected_profile (timer_applet->start_timer_profile_manager_widget);
		g_assert(selected_row);
		timer_applet_get_profile_name_and_duration (timer_applet, selected_row, &preset_name, &timer_duration);
		
		g_assert (preset_name);
		g_assert (timer_duration >= 0);
	}
	
	timer_applet_start_timer (timer_applet, preset_name, timer_duration);

	if (preset_name) {
		g_free (preset_name);
		preset_name = NULL;
	}
}

static void
handle_cancel_clicked (TimerApplet *timer_applet)
{
	g_assert (timer_applet->start_timer_dialog);
	gtk_widget_hide (timer_applet->start_timer_dialog);
}

static void
on_dialog_response (GtkDialog *dialog, gint response_id, TimerApplet *timer_applet)
{
	switch (response_id) {
		case GTK_RESPONSE_OK:
			handle_ok_clicked (timer_applet);
			break;

		case GTK_RESPONSE_CANCEL:
			handle_cancel_clicked (timer_applet);	
			break;
	}
}

static void
on_radio_instant_toggled (GtkRadioButton *radio_button, GtkWidget *controlled_widget)
{
	gboolean radio_active;

	g_object_get (G_OBJECT (radio_button), "active", &radio_active, NULL);
	g_object_set (G_OBJECT (controlled_widget), "sensitive", radio_active, NULL);
	
	{
		GtkWidget *ok_button;
		ok_button = g_object_get_data (G_OBJECT (radio_button), "ok-button");
		g_object_set (G_OBJECT (ok_button), "sensitive", TRUE, NULL);
	}
}

static void
on_radio_use_profile_toggled (GtkRadioButton *radio_button, GtkWidget *controlled_widget)
{
	gboolean radio_active;	

	g_object_get (G_OBJECT (radio_button), "active", &radio_active, NULL);
	g_object_set (G_OBJECT (controlled_widget), "sensitive", radio_active, NULL);

	if (radio_active) {
		/* if this radio button is active, we have to disable the
		 * OK button if there are no profiles (or no profiles selected) in the list
		 */
		 GtkTreeRowReference *selected_row;
		 GtkWidget *profile_manager;
		 
		 profile_manager = g_object_get_data (G_OBJECT (radio_button), "profile-manager");
		 
		 selected_row = profile_manager_get_selected_profile (profile_manager);
		 if (selected_row == NULL) {
		 	/* nothing selected, so disable ok button */
		 	GtkWidget *ok_button;
		 	ok_button = g_object_get_data (G_OBJECT (radio_button), "ok-button");
		 	g_object_set (G_OBJECT (ok_button), "sensitive", FALSE, NULL);

		 }
	}
}

static void
on_row_selected (GtkWidget *profile_manager, gpointer user_data)
{
	TimerApplet *timer_applet;
	timer_applet = (TimerApplet *)user_data;
	
	/* When a row gets highlighted, we want to update the
	 * hours, minutes, and seconds spin buttons.
	 */
	{
		GtkTreeRowReference *selected_profile;
		gchar *name;
		gdouble duration;
		
		g_assert (timer_applet);
		g_assert (timer_applet->start_timer_dialog);
		g_assert (timer_applet->start_timer_profile_manager_widget);
		selected_profile = profile_manager_get_selected_profile (profile_manager);
		
		g_assert (selected_profile);
		
		timer_applet_get_profile_name_and_duration (timer_applet, selected_profile, &name, &duration);
		
		dialog_set_duration (timer_applet->start_timer_dialog, duration);
		
	}
}

static void
on_view_empty (GtkWidget *profile_manager)
{
 	GtkWidget *ok_button;
 	ok_button = g_object_get_data (G_OBJECT (profile_manager), "ok-button");
 	g_object_set (G_OBJECT (ok_button), "sensitive", FALSE, NULL);
}

static void
on_view_not_empty (GtkWidget *profile_manager)
{
 	GtkWidget *ok_button;
 	ok_button = g_object_get_data (G_OBJECT (profile_manager), "ok-button");
 	g_object_set (G_OBJECT (ok_button), "sensitive", TRUE, NULL);
}

static void
on_selection_empty (GtkWidget *profile_manager)
{
 	GtkWidget *ok_button;
 	ok_button = g_object_get_data (G_OBJECT (profile_manager), "ok-button");
 	g_object_set (G_OBJECT (ok_button), "sensitive", FALSE, NULL);
}

static void
on_selection_not_empty (GtkWidget *profile_manager)
{
 	GtkWidget *ok_button;
 	ok_button = g_object_get_data (G_OBJECT (profile_manager), "ok-button");
 	g_object_set (G_OBJECT (ok_button), "sensitive", TRUE, NULL);
}

static void
on_profile_added (GtkWidget *profile_manager, gchar *profile_name, gdouble profile_duration, gpointer user_data)
{
	GtkTreeRowReference *new_row_ref;
	new_row_ref = timer_applet_add_preset (profile_name, profile_duration);
	profile_manager_select_row (profile_manager, new_row_ref);
}

static void
on_profile_edited (GtkWidget *profile_manager, GtkTreeRowReference *row_ref, gchar *profile_name, gdouble profile_duration, gpointer user_data)
{
	timer_applet_update_preset (row_ref, profile_name, profile_duration);
	
	// Cause update of spin buttons
	profile_manager_select_row (profile_manager, row_ref);
}

static void
on_profile_deleted (GtkWidget *profile_manager, GtkTreeRowReference *row_ref, gpointer user_data)
{
	timer_applet_remove_preset (row_ref);
}

void
start_timer_dialog_create (TimerApplet *timer_applet)
{
	GtkWidget *dialog;
	GtkWidget *main_vbox;
	GtkWidget *ok_button;
	
	dialog = gtk_dialog_new_with_buttons (_("Start Timer"), NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
						NULL);
	ok_button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("S_tart Timer"), GTK_RESPONSE_OK);
	
	g_object_set (G_OBJECT (dialog),
					"has-separator", FALSE,
					"border-width", 0,
					NULL);
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
	timer_applet->start_timer_dialog = dialog;
	
	gtk_widget_ensure_style (dialog);
	main_vbox = gtk_vbox_new (FALSE, 18);
	g_object_set (G_OBJECT (main_vbox), "border-width", 12, NULL);
	gtk_widget_show (main_vbox);
	g_object_set (G_OBJECT (GTK_DIALOG (dialog)->vbox), "border-width", 0, NULL);
	g_object_set (G_OBJECT (GTK_DIALOG (dialog)->action_area), "border-width", 12, NULL);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), main_vbox, TRUE, TRUE, 0);
	
	
	// fill in main_vbox
	{
		GtkWidget *radio_label;
		GtkWidget *radio_instant;
		GtkWidget *radio_use_profile;
		
		radio_label = gtk_label_new (_("<span weight=\"bold\">E_nter a timer duration</span>"));
		g_object_set (G_OBJECT (radio_label),
							"use-markup", TRUE,
							"use-underline", TRUE,
							NULL);
		radio_instant = gtk_radio_button_new (NULL);
		gtk_container_add (GTK_CONTAINER (radio_instant), radio_label);
		gtk_widget_show (radio_label);
		
		radio_label = gtk_label_new (_("<span weight=\"bold\">_Use a timer preset</span>"));
		g_object_set (G_OBJECT (radio_label),
							"use-markup", TRUE,
							"use-underline", TRUE,
							NULL);
		radio_use_profile = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_instant));
		gtk_container_add (GTK_CONTAINER (radio_use_profile), radio_label);
		gtk_widget_show (radio_label);
										
		g_object_set_data (G_OBJECT (radio_instant), "ok-button", ok_button);
		g_object_set_data (G_OBJECT (radio_use_profile), "ok-button", ok_button);

		timer_applet->start_timer_radio_instant = radio_instant;
		
		// Add section for first radio button
		{
			GtkWidget *instant_group;
			GtkWidget *duration_chooser;
			
			instant_group = create_group_box (radio_instant);
			duration_chooser = create_duration_chooser ();
			
			{
				GtkWidget *hours_spin;
				GtkWidget *minutes_spin;
				GtkWidget *seconds_spin;
				
				hours_spin = duration_chooser_get_hours_spin (duration_chooser);
				minutes_spin = duration_chooser_get_minutes_spin (duration_chooser);
				seconds_spin = duration_chooser_get_seconds_spin (duration_chooser);
				
				g_object_set_data (G_OBJECT (dialog), "spin-hours", hours_spin);
				g_object_set_data (G_OBJECT (dialog), "spin-minutes", minutes_spin);
				g_object_set_data (G_OBJECT (dialog), "spin-seconds", seconds_spin);
			}
			
			group_box_add_row (instant_group, duration_chooser);
			gtk_widget_show (duration_chooser);
			
			gtk_box_pack_start (GTK_BOX (main_vbox), instant_group, FALSE, FALSE, 0);
			gtk_widget_show (instant_group);
			
			g_signal_connect (G_OBJECT (radio_instant), "toggled", G_CALLBACK (on_radio_instant_toggled), duration_chooser);
		}
		
		// Add section for second radio button
		{
			GtkWidget *use_profile_group;
			GtkWidget *profile_manager_widget;
			
			use_profile_group = create_group_box (radio_use_profile);
			
			
			profile_manager_widget = profile_manager_new (GTK_TREE_MODEL (timer_applet_get_presets_list ()));
			timer_applet->start_timer_profile_manager_widget = profile_manager_widget;
			
			profile_manager_connect_row_selected (profile_manager_widget, on_row_selected, timer_applet);
			profile_manager_set_profile_handlers (profile_manager_widget, on_profile_added, NULL,
											on_profile_edited, NULL, on_profile_deleted, NULL);
			g_object_set_data (G_OBJECT (profile_manager_widget), "ok-button", ok_button);
			profile_manager_set_view_change_handlers (profile_manager_widget, on_view_empty, on_view_not_empty);
			profile_manager_set_selection_change_handlers (profile_manager_widget, on_selection_empty, on_selection_not_empty);
			g_object_set_data (G_OBJECT (radio_use_profile), "profile-manager", profile_manager_widget);
			{
				GtkWidget *profile_manager_tree_view;
				profile_manager_tree_view = profile_manager_get_tree_view (profile_manager_widget);
				g_signal_connect_swapped (G_OBJECT (profile_manager_tree_view), "row-activated", G_CALLBACK (gtk_button_clicked), ok_button);
			}
			
			g_assert (timer_applet->start_timer_profile_manager_widget);
			group_box_add_row_full (use_profile_group, profile_manager_widget, TRUE, TRUE);
			
			gtk_widget_show (profile_manager_widget);
			g_object_set (G_OBJECT (profile_manager_widget), "sensitive", FALSE, NULL);
			
			g_signal_connect (G_OBJECT (radio_use_profile), "toggled", G_CALLBACK (on_radio_use_profile_toggled), profile_manager_widget);
			
			
			gtk_box_pack_start (GTK_BOX (main_vbox), use_profile_group, TRUE, TRUE, 0);
			gtk_widget_show (use_profile_group);
		}
	
		gtk_widget_show (radio_instant);
		gtk_widget_show (radio_use_profile);
	}
	
	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (on_dialog_response), timer_applet);
	g_signal_connect (G_OBJECT (dialog), "delete-event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
	
	gtk_widget_show (main_vbox);
}

void
start_timer_dialog_open (TimerApplet *timer_applet)
{
	gdouble remaining_seconds;

	g_assert (timer_applet);
	
	gtk_widget_show (timer_applet->start_timer_dialog);
}
